# -*- coding: utf-8 -*-
#
# Copyright © Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)

"""
spyder.plugins.shortcuts
========================

Shortcuts Plugin.
"""

from spyder.plugins.shortcuts.plugin import Shortcuts

# The following statement is required to be able to grab internal plugins.
PLUGIN_CLASSES = [Shortcuts]
