# -*- coding: utf-8 -*-
#
# Copyright © Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)

"""
Variable Explorer Plugin API.
"""

# Local imports
from spyder.plugins.variableexplorer.widgets.main_widget import (
    VariableExplorerWidgetActions, VariableExplorerWidgetMainToolBarSections,
    VariableExplorerWidgetMenus, VariableExplorerWidgetOptionsMenuSections)
